#ifndef DIARY_UTILS_H
#define DIARY_UTILS_H

#include <regex.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <pthread.h>
#include <wordexp.h>
#include <stdbool.h>
#include <ncurses.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/tree.h>

#define CAL_WIDTH 21
#define ASIDE_WIDTH 4
#define MAX_MONTH_HEIGHT 6

/* Create simple debug messsages to stderr
 * https://stackoverflow.com/questions/1644868/define-macro-for-debug-printing-in-c
 */
#define debug(fmt, ...) \
    fprintf(stderr, "%s:%d:%s(): " fmt, __FILE__, \
                     __LINE__, __func__, __VA_ARGS__); \
    fprintf(stderr, "\n");

void update_date(WINDOW* header, struct tm* curs_date);
bool date_has_entry(const char* dir, size_t dir_size, const struct tm* i);
char* fold(const char* str);
char* unfold(const char* str);
char* extract_ical_field(const char* ical, char* key, long* start_pos, bool multline);
char* extract_xml_content(const char* xml, const char* xpathExpression, WINDOW* w, pthread_t* p);
char* expand_path(const char* str);
char* strrstr(char *haystack, char *needle);
void fpath(const char* dir, size_t dir_size, const struct tm* date, char** rpath, size_t rpath_size);
bool go_to(WINDOW* calendar, WINDOW* aside, time_t date, int* cur_pad_pos, struct tm* curs_date, struct tm* cal_start, struct tm* cal_end);
void* show_progress(void* vargp);
void show_info(WINDOW* w, char* msg, pthread_t* p);

typedef struct
{
    // Path that holds the journal text files
    char* dir;
    // Number of years to show before/after todays date
    int range;
    // 7 = Sunday, 1 = Monday, ..., 6 = Saturday
    int weekday;
    // 2020-12-31
    char* fmt;
    // Text formatting command/utility fmt,
    // default width 75 chars (-w75), do not refill lines (-s)
    char* fmt_cmd;
    // Don't use pty, use ncurses for preview
    bool no_pty;
    // Don't listen for mouse events
    bool no_mouse;
    // Editor to open journal files with
    char* editor;
    // CalDAV calendar to synchronize
    char* caldav_calendar;
    // CalDAV server URI
    char* caldav_server;
    // CalDAV username
    char* caldav_username;
    // CalDAV password
    char* caldav_password;
    // OAuth command to fetch access token
    char* oauth_eval_cmd;
} config;

extern config CONFIG;

#endif
